@@ -15,6 +15,7 @@
 #define RS256_TOKEN_INVALID_PAYLOAD_B64 "eyJhbGciOiJSUzI1NiIsImtpZCI6IjEifQ.;error;.GHUYpRZXXKm9rUgJsAgR9-mE-dcqVe802BaVz1jy4Z77QA8jwnPkpjbMAWHQuhmJF7oLsKTlJzn2lI5FM7fDq0V70vC50xvz_eu5KoUveje_Swxo-J2S7Cb64M1qT7X6px8J0sxNnkc07WgKZ8XkSZStbhh_H2z5wRjWC_q0rN8XZE6Bh7K0uW08EVCSNk-fRjD84MVHfDNShOi5iYXbDUnsE_uLxG4uMBu9uUPfq8J3mkqbux26omQqy33fyj-6mCHi50OSV6QoUpJl5apAdSOGd5Tl6lgUKHUBn8ykvJkzIjwJWJWYfiNmGbP7JtCNg4a1f4Jdzn3yIXm824UAiPbRsGmdbTJ7d___eeqXkSOc3rrXBihWByZk0EDt_Zc8enLOC9FhfxQ095n6DUlk1NLb1lq-pwaWUXyqVAp66fMNeltZMFVaTUNaZNCj6woXETJWFwquJLVFFTgQRE-HeUsQ2QGhc3wmqqACghrtKAz1om1Aer1OhZDxhGtfosRIgL2x3JCRkn93SXS3gZTgRL4Uvzc_zqdRqDuHNP1d-j4fg38CBTRFa2mkh1PZX3O7f669c0-ANp0ttqKOoS_6N1Pq8KO-IXfghbXIHLLQyffF78OC_e0QjJODXYPfOBIbh_RsRRMrjTcE_wpeUr_L8KCaKT81a2houfwx_tTYnus"
 #define RS256_TOKEN_INVALID_SIGNATURE "eyJhbGciOiJSUzI1NiIsImtpZCI6IjEifQ.VGhlIHRydWUgc2lnbiBvZiBpbnRlbGxpZ2VuY2UgaXMgbm90IGtub3dsZWRnZSBidXQgaW1hZ2luYXRpb24u.GHUY6RZXXKm9rUgJsAgR9-mE-dcqVe802BaVz1jy4Z77QA8jwnPkpjbMAWHQuhmJF7oLsKTlJzn2lI5FM7fDq0V70vC50xvz_eu5KoUveje_Swxo-J2S7Cb64M1qT7X6px8J0sxNnkc07WgKZ8XkSZStbhh_H2z5wRjWC_q0rN8XZE6Bh7K0uW08EVCSNk-fRjD84MVHfDNShOi5iYXbDUnsE_uLxG4uMBu9uUPfq8J3mkqbux26omQqy33fyj-6mCHi50OSV6QoUpJl5apAdSOGd5Tl6lgUKHUBn8ykvJkzIjwJWJWYfiNmGbP7JtCNg4a1f4Jdzn3yIXm824UAiPbRsGmdbTJ7d___eeqXkSOc3rrXBihWByZk0EDt_Zc8enLOC9FhfxQ095n6DUlk1NLb1lq-pwaWUXyqVAp66fMNeltZMFVaTUNaZNCj6woXETJWFwquJLVFFTgQRE-HeUsQ2QGhc3wmqqACghrtKAz1om1Aer1OhZDxhGtfosRIgL2x3JCRkn93SXS3gZTgRL4Uvzc_zqdRqDuHNP1d-j4fg38CBTRFa2mkh1PZX3O7f669c0-ANp0ttqKOoS_6N1Pq8KO-IXfghbXIHLLQyffF78OC_e0QjJODXYPfOBIbh_RsRRMrjTcE_wpeUr_L8KCaKT81a2houfwx_tTYnus"
 #define RS256_TOKEN_INVALID_DOTS "eyJhbGciOiJSUzI1NiIsImtpZCI6IjEifQVGhlIHRydWUgc2lnbiBvZiBpbnRlbGxpZ2VuY2UgaXMgbm90IGtub3dsZWRnZSBidXQgaW1hZ2luYXRpb24u.GHUYpRZXXKm9rUgJsAgR9-mE-dcqVe802BaVz1jy4Z77QA8jwnPkpjbMAWHQuhmJF7oLsKTlJzn2lI5FM7fDq0V70vC50xvz_eu5KoUveje_Swxo-J2S7Cb64M1qT7X6px8J0sxNnkc07WgKZ8XkSZStbhh_H2z5wRjWC_q0rN8XZE6Bh7K0uW08EVCSNk-fRjD84MVHfDNShOi5iYXbDUnsE_uLxG4uMBu9uUPfq8J3mkqbux26omQqy33fyj-6mCHi50OSV6QoUpJl5apAdSOGd5Tl6lgUKHUBn8ykvJkzIjwJWJWYfiNmGbP7JtCNg4a1f4Jdzn3yIXm824UAiPbRsGmdbTJ7d___eeqXkSOc3rrXBihWByZk0EDt_Zc8enLOC9FhfxQ095n6DUlk1NLb1lq-pwaWUXyqVAp66fMNeltZMFVaTUNaZNCj6woXETJWFwquJLVFFTgQRE-HeUsQ2QGhc3wmqqACghrtKAz1om1Aer1OhZDxhGtfosRIgL2x3JCRkn93SXS3gZTgRL4Uvzc_zqdRqDuHNP1d-j4fg38CBTRFa2mkh1PZX3O7f669c0-ANp0ttqKOoS_6N1Pq8KO-IXfghbXIHLLQyffF78OC_e0QjJODXYPfOBIbh_RsRRMrjTcE_wpeUr_L8KCaKT81a2houfwx_tTYnus"
+#define RS256_TOKEN_EMPTY_SIGNATURE "eyJhbGciOiJSUzI1NiIsImtpZCI6IjEifQ.VGhlIHRydWUgc2lnbiBvZiBpbnRlbGxpZ2VuY2UgaXMgbm90IGtub3dsZWRnZSBidXQgaW1hZ2luYXRpb24u."
 
 const char jwk_pubkey_rsa_str[] = "{\"kty\":\"RSA\",\"n\":\"ALrIdhuABv82Y7K1-LJCXRy1LVdmK9IAHwmmlI-HnOrFeEsSwuCeblUgEpqz_mj7lLtZN0Gnlz-7U0hOpGCeOYXRMn8184YismuCS5PYe0"
                                   "Jfot0kMumF2IOBV94AGBSeWQcK8J-Ed3X-rkR9vovv8gXhKyRDQH4mon_cPwtdCi2PScnRlkvlOjYkib9m0QQqpvjmcd02s8BYtakRVRva2mQT_dCvRYvM4Tb5yvvRM7Iz3"
@@ -267,6 +268,15 @@ START_TEST(test_rhonabwy_verify_token_invalid)
   r_jws_free(jws);
   r_jwk_free(jwk_pubkey);
   
+  ck_assert_int_eq(r_jws_init(&jws), RHN_OK);
+  ck_assert_int_eq(r_jwk_init(&jwk_pubkey), RHN_OK);
+  ck_assert_int_eq(r_jws_parse(jws, RS256_TOKEN_EMPTY_SIGNATURE, 0), RHN_ERROR_PARAM);
+  ck_assert_int_eq(R_JWA_ALG_RS256, r_jws_get_alg(jws));
+  ck_assert_int_eq(r_jwk_import_from_json_str(jwk_pubkey, jwk_pubkey_rsa_no_alg_str), RHN_OK);
+  ck_assert_int_eq(r_jws_verify_signature(jws, jwk_pubkey, 0), RHN_ERROR_INVALID);
+  r_jws_free(jws);
+  r_jwk_free(jwk_pubkey);
+  
   ck_assert_int_eq(r_jws_init(&jws), RHN_OK);
   ck_assert_int_eq(r_jwk_init(&jwk_pubkey), RHN_OK);
   ck_assert_int_eq(r_jws_parse(jws, RS256_TOKEN, 0), RHN_OK);
