@@ -48,7 +48,7 @@
 #include <yder.h>
 #include <rhonabwy.h>
 
-#define _RNBYC_VERSION_ "1.1.7"
+#define _RNBYC_VERSION_ "1.1.8"
 
 #define R_RSA_DEFAULT_SIZE 4096
 #define R_OCT_DEFAULT_SIZE 128
@@ -364,7 +364,7 @@ static int jwk_generate(jwks_t * jwks_privkey, jwks_t * jwks_pubkey, json_t * j_
   return ret;
 }
 
-static int jwks_parse_str(jwks_t * jwks_priv, jwks_t * jwks_pub, const char * in, const char * kid, int x5u_flags) {
+static int jwks_parse_str(jwks_t * jwks_priv, jwks_t * jwks_pub, const char * in, const char * kid, const char * alg, int x5u_flags) {
   jwks_t * jwks = NULL;
   jwk_t * jwk = NULL;
   int ret, key_type;
@@ -388,6 +388,9 @@ static int jwks_parse_str(jwks_t * jwks_priv, jwks_t * jwks_pub, const char * in
         if (kid != NULL) {
           r_jwk_set_property_str(jwk, "kid", kid);
         }
+        if (alg != NULL) {
+          r_jwk_set_property_str(jwk, "alg", alg);
+        }
         if (r_jwk_key_type(jwk, NULL, x5u_flags) & R_KEY_TYPE_PUBLIC && jwks_pub != NULL) {
           r_jwks_append_jwk(jwks_pub, jwk);
         } else {
@@ -398,6 +401,9 @@ static int jwks_parse_str(jwks_t * jwks_priv, jwks_t * jwks_pub, const char * in
         if (kid != NULL) {
           r_jwk_set_property_str(jwk, "kid", kid);
         }
+        if (alg != NULL) {
+          r_jwk_set_property_str(jwk, "alg", alg);
+        }
         if (r_jwk_key_type(jwk, NULL, x5u_flags) & R_KEY_TYPE_PUBLIC && jwks_pub != NULL) {
           r_jwks_append_jwk(jwks_pub, jwk);
         } else {
@@ -408,6 +414,9 @@ static int jwks_parse_str(jwks_t * jwks_priv, jwks_t * jwks_pub, const char * in
         if (kid != NULL) {
           r_jwk_set_property_str(jwk, "kid", kid);
         }
+        if (alg != NULL) {
+          r_jwk_set_property_str(jwk, "alg", alg);
+        }
         if (r_jwk_key_type(jwk, NULL, x5u_flags) & R_KEY_TYPE_PUBLIC && jwks_pub != NULL) {
           r_jwks_append_jwk(jwks_pub, jwk);
         } else {
@@ -418,6 +427,9 @@ static int jwks_parse_str(jwks_t * jwks_priv, jwks_t * jwks_pub, const char * in
         if (kid != NULL) {
           r_jwk_set_property_str(jwk, "kid", kid);
         }
+        if (alg != NULL) {
+          r_jwk_set_property_str(jwk, "alg", alg);
+        }
         if (r_jwk_key_type(jwk, NULL, x5u_flags) & R_KEY_TYPE_PUBLIC && jwks_pub != NULL) {
           r_jwks_append_jwk(jwks_pub, jwk);
         } else {
@@ -440,7 +452,7 @@ static int jwk_stdin(jwks_t * jwks_priv, jwks_t * jwks_pub, json_t * j_element,
   int ret;
 
   if (in != NULL) {
-    ret = jwks_parse_str(jwks_priv, jwks_pub, in, json_string_value(json_object_get(j_element, "kid")), x5u_flags);
+    ret = jwks_parse_str(jwks_priv, jwks_pub, in, json_string_value(json_object_get(j_element, "kid")), json_string_value(json_object_get(j_element, "alg")), x5u_flags);
   } else {
     ret = EIO;
   }
@@ -453,7 +465,7 @@ static int jwk_file(jwks_t * jwks_priv, jwks_t * jwks_pub, json_t * j_element, i
   int ret;
 
   if (in != NULL) {
-    ret = jwks_parse_str(jwks_priv, jwks_pub, in, json_string_value(json_object_get(j_element, "kid")), x5u_flags);
+    ret = jwks_parse_str(jwks_priv, jwks_pub, in, json_string_value(json_object_get(j_element, "kid")), json_string_value(json_object_get(j_element, "alg")), x5u_flags);
   } else {
     ret = EIO;
   }
@@ -674,17 +686,19 @@ static int parse_token(const char * token, int indent, int x5u_flags, const char
       }
       if (r_jwks_size(jwks_privkey)) {
         if (type == R_JWT_TYPE_ENCRYPT) {
-          if (r_jwt_decrypt(jwt, NULL, x5u_flags) == RHN_OK) {
+          int rc = r_jwt_decrypt(jwt, NULL, x5u_flags);
+          if (rc == RHN_OK) {
             fprintf(stdout, "Token payload decrypted\n");
           } else {
-            fprintf(stderr, "Unable to decrypt payload %d\n", r_jwt_decrypt(jwt, NULL, x5u_flags));
+            fprintf(stderr, "Unable to decrypt payload %d\n", rc);
             ret = EINVAL;
           }
         } else if (type == R_JWT_TYPE_NESTED_ENCRYPT_THEN_SIGN || type == R_JWT_TYPE_NESTED_SIGN_THEN_ENCRYPT) {
-          if (r_jwt_decrypt_nested(jwt, NULL, x5u_flags) == RHN_OK) {
+          int rc = r_jwt_decrypt_nested(jwt, NULL, x5u_flags);
+          if (rc == RHN_OK) {
             fprintf(stdout, "Token payload decrypted\n");
           } else {
-            fprintf(stderr, "Unable to decrypt payload %d\n", r_jwt_decrypt(jwt, NULL, x5u_flags));
+            fprintf(stderr, "Unable to decrypt nested payload %d\n", rc);
             ret = EINVAL;
           }
         }
@@ -703,6 +717,18 @@ static int parse_token(const char * token, int indent, int x5u_flags, const char
         printf("%s\n", str_value);
         o_free(str_value);
         json_decref(j_value);
+        // also print jwt header
+        if ((type == R_JWT_TYPE_NESTED_ENCRYPT_THEN_SIGN ||
+             type == R_JWT_TYPE_NESTED_SIGN_THEN_ENCRYPT ||
+             type == R_JWT_TYPE_ENCRYPT) &&
+            jwt->jws)
+        {
+          j_value = r_jws_get_full_header_json_t(jwt->jws);
+          str_value = json_dumps(j_value, (size_t)(JSON_INDENT(indent)|JSON_SORT_KEYS));
+          printf("%s\n", str_value);
+          o_free(str_value);
+          json_decref(j_value);
+        }
       }
       if (show_claims) {
         str_value = NULL;
@@ -906,6 +932,7 @@ int main (int argc, char ** argv) {
   json_t * j_arguments = json_array();
   unsigned long bits = 0;
   int indent = 2;
+  jwa_alg j_alg = R_JWA_ALG_NONE;
 
   do {
     next_option = getopt_long(argc, argv, short_options, long_options, NULL);
@@ -1005,20 +1032,35 @@ int main (int argc, char ** argv) {
         split_keys = 1;
         break;
       case 'a':
-        if (action == R_ACTION_JWKS_OUT) {
-          json_object_set_new(json_array_get(j_arguments, json_array_size(j_arguments)-1), "alg", json_string(optarg));
+        if ((j_alg = r_str_to_jwa_alg(optarg)) != R_JWA_ALG_NONE && j_alg != R_JWA_ALG_UNKNOWN) {
+          if (action == R_ACTION_JWKS_OUT) {
+            json_object_set_new(json_array_get(j_arguments, json_array_size(j_arguments)-1), "alg", json_string(optarg));
+          } else {
+            o_free(alg);
+            alg = o_strdup(optarg);
+          }
         } else {
-          o_free(alg);
-          alg = o_strdup(optarg);
+          fprintf(stderr, "--alg: argument invalid: %s\n", optarg);
+          ret = EINVAL;
         }
         break;
       case 'e':
-        o_free(enc);
-        enc = o_strdup(optarg);
+        if (r_str_to_jwa_enc(optarg) != R_JWA_ENC_UNKNOWN) {
+          o_free(enc);
+          enc = o_strdup(optarg);
+        } else {
+          fprintf(stderr, "--enc: argument invalid: %s\n", optarg);
+          ret = EINVAL;
+        }
         break;
       case 'l':
-        o_free(enc_alg);
-        enc_alg = o_strdup(optarg);
+        if ((j_alg = r_str_to_jwa_alg(optarg)) != R_JWA_ALG_NONE && j_alg != R_JWA_ALG_UNKNOWN) {
+          o_free(enc_alg);
+          enc_alg = o_strdup(optarg);
+        } else {
+          fprintf(stderr, "--enc-alg: argument invalid: %s\n", optarg);
+          ret = EINVAL;
+        }
         break;
       case 'n':
         indent = (int)strtol(optarg, NULL, 10);
